var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
// eslint-disable-next-line @typescript-eslint/no-var-requires
const BlueBirdPromise = require("bluebird");
import { readFile } from "fs/promises";
import { statSync } from "fs";
import FormData from "form-data";
import { Mutex } from "async-mutex";
import { inspect } from "util";
import logManager from "../utils/log-manager";
const logger = logManager.getLogger("image-importer");
const mutex = new Mutex();
function importImages(gateway, allImages, imageDirectory, processedSku, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { deleteExisting, imageImportConcurrency } = options;
        const images = BlueBirdPromise.map(allImages, (image) => importImage(gateway, image, imageDirectory, processedSku, { deleteExisting }), {
            concurrency: imageImportConcurrency,
        });
        return BlueBirdPromise.all(images).catch((error) => {
            logger.error(error);
            throw error;
        });
    });
}
function importImage(gateway, image, imageDirectory, processedSku, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { deleteExisting } = options;
        let hasProductBeenDeleted = false;
        const productPage = yield gateway.PCM.Filter({ eq: { sku: image.sku } })
            .With(["main_image"])
            .All()
            .catch((err) => {
            logger.error(`Error occured while getting product ${image.sku}`, err);
            throw new Error(`Unable to get product  ${image.sku}`);
        });
        const product = productPage.data[0];
        if (product) {
            if (deleteExisting && deleteExisting === true) {
                logger.debug(`Deleting existin images for product ${product.attributes.sku} ...`);
                yield mutex.runExclusive(() => __awaiter(this, void 0, void 0, function* () {
                    if (processedSku.includes(product.attributes.sku)) {
                        hasProductBeenDeleted = true;
                    }
                    else {
                        processedSku.push(product.attributes.sku);
                    }
                }));
                if (hasProductBeenDeleted === false) {
                    yield deleteProductImages(gateway, product);
                }
            }
            logger.debug(`Importing images for product ${product.attributes.sku} ...`);
            const basePath = imageDirectory.endsWith("/") ? imageDirectory : imageDirectory.concat("/");
            const fileId = yield uploadFile(gateway, basePath + image.file_name);
            if (fileId && fileId !== "") {
                return createAssociations(gateway, product.id, fileId, image.main_image);
            }
        }
        else {
            logger.warn(`Product ${image.sku.sku} not found.  SKIP`);
            return BlueBirdPromise.resolve(`Product ${image.sku.sku} not found.  SKIP`);
        }
    });
}
function uploadFile(gateway, filename) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(`Uploading ${filename}...`);
        const formData = new FormData();
        const stats = statSync(filename);
        const fileSizeInBytes = stats.size;
        const fileBuffer = yield readFile(filename);
        formData.append("file", fileBuffer, {
            knownLength: fileSizeInBytes,
            filename: filename,
        });
        return gateway.request
            .send("files", "POST", formData.getBuffer(), undefined, undefined, false, "v2", {
            "Content-Type": formData.getHeaders()["content-type"]
        })
            .then((response) => {
            if (response) {
                return response.data.id;
            }
            return BlueBirdPromise.resolve("");
        })
            .catch((error) => {
            logger.warn(inspect(error, false, 5, true));
            logger.warn(`Unable to upload file - FAILURE (${JSON.stringify(error)})`);
        });
    });
}
function createAssociations(gateway, productId, fileId, isMainImage) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(`Associating product ${productId} with image ${fileId}, mainImageFlag=${isMainImage}`);
        yield createFileRelationship(gateway, productId, fileId);
        if (isMainImage) {
            const isThereAlreadyAMainImage = yield doesProductHaveAMainImage(gateway, productId);
            if (isThereAlreadyAMainImage) {
                return updateMainImageRelationship(gateway, productId, fileId);
            }
            else {
                return createMainImageRelationship(gateway, productId, fileId);
            }
        }
        return BlueBirdPromise.resolve(`Associated product ${productId} with image ${fileId}, mainImageFlag=${isMainImage}`);
    });
}
function createFileRelationship(gateway, productId, fileId) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(`"Associated productId ${productId} to fileId ${fileId}`);
        return gateway.PCM.FileRelationships.Create(productId, fileId);
    });
}
function doesProductHaveAMainImage(gateway, productId) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = yield gateway.PCM.MainImageRelationships.Get(productId);
        const isThereAMainImage = response && response.data && response.data.id;
        return isThereAMainImage;
    });
}
function updateMainImageRelationship(gateway, productId, fileId) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(`Updated MAIN IMAGE for productId ${productId} to fileId ${fileId}`);
        return gateway.PCM.MainImageRelationships.Update(productId, fileId);
    });
}
function createMainImageRelationship(gateway, productId, fileId) {
    return __awaiter(this, void 0, void 0, function* () {
        logger.debug(`Created MAIN IMAGE for productId ${productId} to fileId ${fileId}`);
        return gateway.PCM.MainImageRelationships.Create(productId, fileId);
    });
}
function deleteProductImages(gateway, product) {
    var _a, _b, _c, _d, _e;
    return __awaiter(this, void 0, void 0, function* () {
        if ((_c = (_b = (_a = product.relationships) === null || _a === void 0 ? void 0 : _a.main_image) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.id) {
            yield gateway.PCM.MainImageRelationships.Delete(product.id, (_e = (_d = product.relationships) === null || _d === void 0 ? void 0 : _d.main_image) === null || _e === void 0 ? void 0 : _e.data.id)
                .then((response) => {
                var _a, _b;
                logger.debug(`Remove of MainImage Relationship ${product.id}::${(_b = (_a = product.relationships) === null || _a === void 0 ? void 0 : _a.main_image) === null || _b === void 0 ? void 0 : _b.data.id} SUCCESS`);
            })
                .catch((error) => {
                var _a, _b;
                logger.warn(`Remove of MainImage Relationship ${product.id}::${(_b = (_a = product.relationships) === null || _a === void 0 ? void 0 : _a.main_image) === null || _b === void 0 ? void 0 : _b.data.id} FAILURE (${error})`);
            });
        }
        const fileRelationshipList = yield gateway.PCM.FileRelationships.All(product.id);
        const removeRelationsShip = BlueBirdPromise.map(fileRelationshipList.data, (relationShip) => deleteProductMedia(gateway, product.id, relationShip.id), {
            concurrency: 5,
        });
        return BlueBirdPromise.all(removeRelationsShip).catch((error) => {
            logger.error(JSON.stringify(error));
            throw error;
        });
    });
}
function deleteProductMedia(gateway, productId, fileId) {
    return __awaiter(this, void 0, void 0, function* () {
        yield gateway.PCM.FileRelationships.Delete(productId, fileId)
            .then((response) => {
            logger.debug(`Remove of Product-File ${productId}::${fileId} SUCCESS`);
        })
            .catch((error) => {
            logger.warn(`Remove of Profile-File ${productId}::${fileId} FAILURE (${error})`);
        });
        logger.debug(`Removing File ${fileId}...`);
        return gateway.Files.Delete(fileId);
    });
}
module.exports = {
    importImages,
    uploadFile,
};
export default { importImages, uploadFile };
